<?php

namespace Bydedi\Avalara;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\ClientException;
use GuzzleHttp\Message\ResponseInterface;

/**
 * Class TaxRates
 */
class TaxRates
{
    /**
     * The API base URL
     */
    const API_URL = 'https://taxrates.api.avalara.com:{port}';
    const API_PORT = '443';

    /**
     * Error messages
     */
    const ERROR_400A    = 'The postal code you provided was not valid. Please check your postal code before re-trying.';
    const ERROR_400B    = 'One of the values you submitted was empty or the postal code you provied was not valid.';
    const ERROR_401     = 'Your authorization credentials were not provided or were invalid.';
    const ERROR_429     = 'Rate limiting has been exceeded. Try again later.';
    const ERROR_UNKNOWN = 'Unknown error with the tax rates api. Try again later.';

    /**
     * Default country
     */
    const COUNTRY = 'USA';

    /**
     * @var TaxRates|null
     */
    static private $instance = null;

    /**
     * @var string
     */
    protected $apikey;

    /**
     * @var Client
     */
    protected $client;

    /**
     * TaxRates constructor.
     * @param $config
     */
    private function __construct($config)
    {
        $this->apikey = $config['avalara']['apikey'];
        $this->client = new Client([
            'base_url' => [self::API_URL, ['port' => self::API_PORT]],
            'defaults' => [
                'query'   => ['apikey' => $this->apikey, 'country' => 'usa'],
            ]
        ]);
    }

    /**
     * Get instance of the singleton
     * @param $config
     * @return TaxRates
     */
    public static function getInstance(array $config)
    {
        if (!(self::$instance instanceof self)) {
            self::$instance = new self($config);
        }
        return self::$instance;
    }

    /**
     * @param $zipcode
     * @return mixed
     */
    public function getRatesFromZipcode($zipcode)
    {
        $result = null;

        try {
            $response = $this->client->get('/postal', ['query' => ['postal' => $zipcode]]);
            $result = json_decode($response->getBody());
        } catch (ClientException $e) {
            $this->handleResponse($e->getResponse());
        }

        return $result;
    }

    /**
     * @param $address : state, city, postal, street
     * @return mixed
     */
    public function getRatesFromAddress($address)
    {
        $result = null;

        try {
            $response = $this->client->get('/address', ['query' => $address]);
            $result = json_decode($response->getBody());
        } catch (ClientException $e) {
            $this->handleResponse($e->getResponse());
        }

        return $result;
    }

    protected function handleResponse(ResponseInterface $response)
    {
        $statusCode = $response->getStatusCode();
        $path = parse_url($response->getEffectiveUrl(), PHP_URL_PATH);

        if (substr($statusCode, 0, 1) == 4) {
            switch ($statusCode) {
                case 400:
                    if ($path == '/address') {
                        $error = self::ERROR_400B;
                    } else {
                        $error = self::ERROR_400A;
                    }
                    break;
                case 401:
                    $error = self::ERROR_401;
                    break;
                case 429:
                    $error = self::ERROR_429;
                    break;
                default:
                    $error = self::ERROR_UNKNOWN;
                    break;
            }

            throw new \Exception($error);
        }
    }
}
